class eosclient::fuse {

  # repo set up etc.
  # keep this include until all clients drop the deprecated include eosclient::fuse syntax
  include ::eosclient

  # Last argument wins in deep_merge
  $eosclient_sysconfig = deep_merge($::eosclient::_deprecated_eosclient_custom_config, $::eosclient::common_config)
  $_eosclient_sysconfig_legacy = delete($eosclient_sysconfig,'conf')

  $allmounts = keys($::eosclient::instance_config)

  if empty($::eosclient::mounts) {
    $eosmounts = $::eosclient::default_mounts
  } else {
    $eosmounts = $::eosclient::mounts
  }

  $eosmounts_legacy = difference($eosmounts, $::eosclient::eosxd_ready)

  $unmounted = difference($allmounts, $eosmounts)
  eosclient::mount { $unmounted:
    remove => true,
  }

  if $::eosclient::enable_autofs {
    $eospkg = ['eos-fuse-core', 'eos-fusex']
    # eos-fusex is a metapackage for (currently) eos-fusex-core, eos-fusex-selinux
  } else {
    $eospkg = ['eos-fuse', 'eos-fuse-core', 'eos-fusex', 'eos-fuse-sysv']
  }

  if $::eosclient::allow_osrepos {
    Osrepos::Kojitag['eos'] -> Package[$eospkg]
  }

  ensure_packages($eospkg, {'ensure' => $::eosclient::version})

  # rotating by 'size' implictly turns off 'daily' rotation.
  # The augeas lens does something funny with the "daily" entry, don't touch.
  $logrotate_fuse_changes = $::eosclient::fuse_logrotate_size ? {
    ''       => [
                  "set /files/etc/logrotate.d/eos-fuse-logs/rule/rotate ${::eosclient::fuse_logrotate_days}",
                  'rm /files/etc/logrotate.d/eos-fuse-logs/rule/size',
                ],
    default => [
                  "set /files/etc/logrotate.d/eos-fuse-logs/rule/rotate ${::eosclient::fuse_logrotate_days}",
                  "set /files/etc/logrotate.d/eos-fuse-logs/rule/size ${::eosclient::fuse_logrotate_size}",
                ],
  }

  $logrotate_fusex_changes = $::eosclient::fuse_logrotate_size ? {
    ''       => [
                  "set /files/etc/logrotate.d/eos-fusex-logs/rule/rotate ${::eosclient::fuse_logrotate_days}",
                  'rm /files/etc/logrotate.d/eos-fusex-logs/rule/size',
                ],
    default => [
                  "set /files/etc/logrotate.d/eos-fusex-logs/rule/rotate ${::eosclient::fuse_logrotate_days}",
                  "set /files/etc/logrotate.d/eos-fusex-logs/rule/size ${::eosclient::fuse_logrotate_size}",
                ],
  }

  augeas{'eosfuse_logrotate':
    changes => $logrotate_fuse_changes,
    incl    => '/etc/logrotate.d/eos-fuse-logs',
    lens    => 'Logrotate.lns',
    require =>  Package[$eospkg],
  }
  -> augeas{'eosfusex_logrotate':
    changes => $logrotate_fusex_changes,
    incl    => '/etc/logrotate.d/eos-fusex-logs',
    lens    => 'Logrotate.lns',
    require =>  Package[$eospkg],
  }
  -> file{'/etc/sysconfig/eos':
    content => template('eosclient/sysconfig_eos.erb'),
    mode    => '0444',
    owner   => root,
    group   => root,
  }
  -> file{'/eos':
    ensure  => directory,
    mode    => '0755',
    owner   => root,
    group   => root,
    seltype => 'default_t',
  }
  -> file{'/var/run/eosd':
    ensure => directory,
    mode   => '0755',
    owner  => root,
    group  => root,
  }
  -> file{'/var/run/eosd/credentials':
    ensure => directory,
    mode   => '1777',
    owner  => root,
    group  => root,
  }
  -> file{'/var/run/eosd/credentials/store':
    ensure => directory,
    mode   => '1777',
    owner  => root,
    group  => root,
  }
  -> file{'/var/log/eos':
    ensure => directory,
    mode   => '0755',
    owner  => daemon,
    group  => daemon,
  }
  -> file{'/var/log/eos/fuse':
    ensure => directory,
    mode   => '0755',
    owner  => daemon,
    group  => daemon,
  }
  -> file{'/etc/eos':
    ensure => directory,
    mode   => '0755',
    owner  => root,
    group  => root,
  }

  $eosmounts.each |$_mount | {
    # Has to be oneliner for this, can't get slice or filter to do the correct thing.
    # Want to delete everything except for 'conf'
    $slice = { 'conf' => $eosclient::_deprecated_eosclient_custom_config['conf'] }
    if $_mount in $allmounts {
      $_instanceconfig = deep_merge($slice,
                                    $eosclient::_deprecated_eosclient_instance_config[$_mount],$::eosclient::instance_config[$_mount])
    } else{
      fail("There is no default configuration for mount ${_mount}")
    }
    if $_mount in $::eosclient::eosxd_ready {
      $_eosx = true
    } else {
      $_eosx = false
    }

    $_config = deep_merge($::eosclient::common_config,$_instanceconfig)

    eosclient::mount{$_mount:
      config  => $_config,
      eosx    =>  $_eosx,
      require => File['/etc/eos'],
    }
  }

  file {'/usr/local/sbin/eos-cleanup.sh':
    source => 'puppet:///modules/eosclient/eos-cleanup.sh',
    mode   => '0755',
    owner  => root,
    group  => root,
  }
  -> cron {'eos-cleanup':
    command => '/usr/local/sbin/eos-cleanup.sh &> /dev/null',
    minute  => '*/5',
  }

  if $::eosclient::enable_autofs {

    include ::eosclient::autofs
    Eosclient::Mount[$eosmounts] -> Package['autofs']

  } else {

    service{'eosd':
      ensure    => 'running',
      enable    => true,
      hasstatus => true,
      require   => [ Eosclient::Mount[$eosmounts] ],
    }

    # LEMON used the 'eosd' init script return code (0 = running, 1=dead+locked, 2=stopped)
    # no longer applicable, and anyway only ever for 'eosd'
    if $eosclient::enable_lemon {
      deprecation('eosclient::enable_lemon', 'The hiera key "eosclient::enable_lemon" no longer does anything, please remove from your configuration')
    }
  }
}
